#ifndef TLV_H
#define TLV_H

#include <QtCore>
#include <hello.h>
#include <node.h>
#include <bundle.h>

/*! \brief This class converts datagrams to objects and vice versa.
 * 
 */
class TLV : public QObject
{

Q_OBJECT 
	public:
		TLV(QObject *parent = 0);
		int fileOption;
		QDir dir;

	signals:
		/*! \brief
		 * Sends a datagram to the appropriate Connection object, 
		 * which in turn passes it on as a DataPacket to 
		 * NeighborAwareness where it is sent out on a socket.
		 */
		void sendDatagram(QByteArray);
	
		/*! \brief
		 * Emits a Hello message to the appropriate Connection object.
		 */
		void sendHello(Hello);
		
		/*! \brief
		 * Emits a list with id<-> name mappings to the appropriate 
		 * Conenction object.
		 */
		void sendDictionary(QList<Node>);
		
		/*! \brief
		 * Emits a list of id<->probability mappings to the 
		 * appropriate Connection object.
		 */
		void sendRIB(QList<Node>);
		
		/*! \brief
		 * Emits a bundle offer to the appropriate Connection object.
		 */
		void sendBundleOffer(QList<Bundle>);
		
		/*! \brief
		 * Emits a bundle request to the appropriate Connection object.
		 */
		void sendBundleRequest(QList<Bundle>);
		
		//! Emits a list of bundles containing data.
		/*!
		 * Connected to Connection::receiveBundles.
		 */
		void sendBundleData(QList<Bundle>);
		
	public slots:
	/*! \brief
	 * Builds a hello datagram and emits the signal sendDatagram.
	 * @param hello the Hello object to build a datagram of.
	 */
	void receiveHello(Hello);
		
	/*! \brief
	 * Classifies incoming Datagrams, creates objects from them and emits 
	 * the appropriate signals.
	 * @param datagram the data to be parsed.
	 */
	void receiveDatagram(QByteArray);
		
	/*! \brief
	 * Builds a datagram containing a dictionary, that is a mapping 
	 * between node ids and names. The datagram is emitted by the 
	 * sendDatagram signal.
	 * @param the list of nodes to create the dictionary from.
	 */
	void createDictionary(QList<Node>);
	
	/*! \brief
	 * Builds a datagram containing a RIB, that is a mapping between node 
	 * ids and their delivery predictabilities. Emits the sendDatagram 
	 * signal.
	 * @param the list of nodes to create the RIB from.
	 */
	void createRIB(QList<Node>);
	
	/*! \brief
	 * Builds a bundle offer datagram. That is a list of all bundles this 
	 * node is carrying. Emits the sendDatagram signal.
	 * @param the list of bundles to create the offer from.
	 */
	void createBundleOffer(QList<Bundle>);
	
	/*! \brief
	 * Builds a bundle request datagram, that is a datagram containing a 
	 * list of the bundles this node would like receive. Emits the 
	 * sendDatagram signal.
	 * @param a list of the bundles to be requested.
	 */
	void createBundleRequest(QList<Bundle>);
	
	/*! \brief
	 * Builds a datagram containing all bundles to be transferred to a 
	 * neighbor node. Emits the sendDatagram signal.
	 * @param the list of bundles to send to the neighbor node.
	 */
	void createBundleData(QList<Bundle>);
};

#endif
